
#ifndef NESSUS_NESSUSLIB_H__
#define NESSUS_NESSUSLIB_H__


/*
 * Arglist definition and defines
 */
#include <sys/types.h>
struct arglist {
	char * name;
	int type;
	void * value;
	long length;
	struct arglist * next;
	};
	
#define ARG_STRING 	1
#define ARG_PTR 	2
#define ARG_INT 	3
#define ARG_ARGLIST 	4





/*
 * Plugin standard function templates
 */

typedef int(*plugin_init_t)(struct arglist *);
typedef int(*plugin_run_t)(struct arglist *);      





/*
 * Network-related functions
 */

/* Plugin specific network functions */
int open_sock_tcp(struct arglist * , unsigned int );
int open_sock_udp(struct arglist * , unsigned int );
int open_sock_option(struct arglist * , unsigned int , int , int );
short is_cgi_installed(struct arglist * , char *);

/* Additional functions -- should not be used by the plugins */
int open_sock_tcp_hn(char * , unsigned int );
int open_sock_opt_hn(char * , unsigned int , int , int );
struct in_addr nn_resolve (char *); 






/* 
 * Management of the arglists --should not be used directly by
 * the plugins
 */

void arg_add_value(struct arglist *, char *, int, long, void *);	   
int arg_set_value(struct arglist *, char *, long, void *);	
int arg_set_type(struct arglist *, char *, int);
void * arg_get_value(struct arglist *, char *);
int arg_get_type(struct arglist *, char *);
void arg_dump(struct arglist *, int);





/*
 * Arglist management at plugin-level
 */
 
void plug_set_name(struct arglist *, char *);
void plug_set_launch(struct arglist *, int);
void plug_set_summary(struct arglist *, char *);
void plug_set_description(struct arglist *, char *);
void plug_set_category(struct arglist *, int);
void plug_set_copyright(struct arglist *, char *);
void plug_set_family(struct arglist * , char * );
char * plug_get_family(struct arglist * );
void plug_add_dep(struct arglist *, char *, char *);
void plug_add_targetos(struct arglist *, char *);
void plug_add_targetos_family(struct arglist *, char *);
void plug_add_port(struct arglist *, int);
struct arglist * plug_get_targetos(struct arglist *);
struct arglist * plug_get_targetos_family(struct arglist *);
char * plug_get_hostname(struct arglist *);
void plug_add_host(struct arglist *, struct arglist *);
int plug_get_launch(struct arglist *);




/*
 * Reporting functions
 */
 
/* Plugin-specific : */
void post_hole(struct arglist *, int, char *);
void post_info(struct arglist *, int, char *);

/* Client-specific : */
void l_post_hole(struct arglist *, int, char *);
void l_post_info(struct arglist *, int, char *);







/* 
 * Management of the portlists
 */


void host_add_port(struct arglist *, int, int, char *);
int host_get_port_state(struct arglist *, int);
/* Not implemented
char * host_get_port_banner(struct arglist *, int);
*/






/*
 * Miscellaneous functions
 */
 
struct in_addr * plug_get_host_ip(struct arglist *);
char * plug_get_host_name(struct arglist *);
char * get_preference(struct arglist *, char *);





/*
 * Replacement for system related functions
 */
void * emalloc(unsigned long);
void efree(void *);
size_t estrlen(char *, size_t);





/* 
 * Misc. defines
 */
 
/* Actions types of the plugins */
#define ACT_DENIAL 3
#define ACT_ATTACK 2
#define ACT_GATHER_INFO 1

 
#endif
