/* Hostloop -- the Hostloop Library
 * Copyright (C) 1998 Renaud Deraison
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdlib.h>
#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <unistd.h>
#include <pwd.h>
#include <fcntl.h>
#include <sys/mman.h>

#ifdef __GLIBC__
#include <fnmatch.h>
#endif

#include "parser.h"

extern void *emalloc(unsigned long);
extern void efree(void*);



#define eprintfV p_eprintfV
/*#define malloc(x) p_malloc(x)*/
#define resolve(x) p_resolve(x)

struct parses_t * thd_parses;
struct parses_t * do_parse(char * name, struct parses_t * p);
short get_hostname_attributes(char * name);

static char * get_parses_name();

#ifndef __GLIBC__
int fnmatch(const char *__pattern, const char *__string,
                         int __flags);    
                         
                         

int fnmatch(const char *__pattern, const char *__string,
                         int __flags)
{
#error "fnmatch() not implemented -- can't compile"
	return(-1);
}                   
                       
#endif




static char * get_parses_name()
{
  char * tmp;
  struct passwd * p;
  
  p = getpwuid(getuid());
  endpwent();
  if(!p)return(NULL);
  tmp = malloc(strlen(p->pw_dir) + 11);
  sprintf(tmp, "%s/.thdrules", p->pw_dir);
  return(tmp); 
}


void init_parses()
{
  int f;
  struct parses_t * p ;
  char * fn = get_parses_name();
  
  if(!fn)return;
  p = thd_parses = emalloc(sizeof(struct parses_t));
  f = open(fn, O_RDONLY); 
  if(f<0)
    {
      char * buf = emalloc(200);
      f = open(fn, O_CREAT | O_RDWR,  0660);
      chown(fn, getuid(), getgid());
      close(f);
      f = open(fn, O_RDWR);
      sprintf(buf, 
"#TriHaDes rules\n#cd doc/sample.thdrules for details\nn:localhost.*\n\n");
      write(f, buf, strlen(buf));
      efree(buf);
      close(f);
      if(!p){printf("ERROR : p == null !\n");exit(0);}
      p->name = emalloc(12);
      sprintf(p->name, "localhost.*");
      p->category = p_DONT_TEST;
      p->next = NULL;
    }
  else
    {
      char * ptr;
      int flag = 1;
      int i = 0;
      
      
      ptr = mmap(NULL, 16384, PROT_READ, MAP_SHARED, f, 0);
      if((int)ptr==-1){perror("mmap ");return;}
      
      
      while(flag)
	{
	int j=0;
	char * buf = emalloc(255);
	while(ptr[i]!=0)
	{
		if((buf[j++]=ptr[i++])=='\n')break;
	}
	if(ptr[i]==0)flag = 0;
	else 
	{
	buf[j-1]=0;
	if(buf[0]!='#')
	    {
	      switch(buf[0])
		{
		case 'n':
		  p->category = p_DONT_TEST;
		  break;
		case 'd':
		  p->category = p_DONT_DNS;
		  break;
		case 's':
		  p->category = p_DONT_SM;
		  break;
		case 'y':
		  p->category = p_DO_TEST;
		  break;
		default : 
		  p->category = -1;
		  fprintf(stderr, "Parse error in %s : \"%s\"\n",fn,  buf);
		}
	      if(p->category != -1)
		{
		  buf+=2;
		  p->name = emalloc(strlen(buf) + 1);
		  sprintf(p->name, "%s", buf);
		  p->next = emalloc(sizeof(struct parses_t));
		  bzero(p->next, sizeof(struct parses_t));
		  p = p->next;
		  buf-=2;
		}
	    }
	  }
	  efree(buf);
	}
      close(f);
      munmap(ptr, 16384);
    }
}



struct parses_t * do_parse(char * name, struct parses_t * p)
{
  char good = 1;
  
  while(p && p->name && good)
    {
      if(!fnmatch(p->name, name, 0))good = 0;
      else p = p->next;
    }
  return(good ? NULL:p);
}


short get_hostname_attributes(char * name)
{
  struct parses_t * p = thd_parses;
  short a=0;
  
  while(p && p->name)
   {
    p = do_parse(name, p);
    if(p)
    {
    a += (a & p->category)?0:p->category;
    if(p)p = p->next;
    }
   }
  return(a);
} 

	
		
	
	  
