/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
 
#include <config.h>
#include <nessuslib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <gtk/gtk.h>

#include "attack.h"
#include "xstuff.h"
#include "comm.h"
#include "auth.h"
#include "parser.h"             
#include "report.h"
 
struct monitor_data {
  int tag;
  struct arglist * hosts;
  GtkWidget * box;
  GtkWidget * window;
  GtkWidget * list;
	};
extern struct arglist * hosts;
void monitor_input_callback(struct monitor_data *, int, int);
void monitor_add_host(struct monitor_data *, char *, int);
void monitor_stop_test(GtkWidget *, char *);

void monitor_dialog_setup(victim)
     char * victim;
{
  struct monitor_data data;
  GtkWidget * scrolled_window; 
 
  hosts = emalloc(sizeof(struct arglist));
  data.hosts = hosts;
  data.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(data.window), "delete_event",
		     (GtkSignalFunc)close_display,NULL);
  gtk_window_set_title(GTK_WINDOW(data.window), "Nessus portscanning/attack status");
  gtk_container_border_width(GTK_CONTAINER(data.window), 10);
  
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_container_border_width (GTK_CONTAINER (scrolled_window), 10);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
				
  gtk_container_border_width(GTK_CONTAINER(scrolled_window), 10);   
  gtk_widget_set_usize(scrolled_window, 500,300); 
  gtk_container_add(GTK_CONTAINER(data.window), scrolled_window);
  
  
  data.list = gtk_list_new();
  gtk_container_add(GTK_CONTAINER(scrolled_window), data.list);
  gtk_widget_show(data.list);
  gtk_widget_show (scrolled_window);
  gtk_widget_show(data.window);
                       
  data.tag = gdk_input_add (0, GDK_INPUT_READ,
			    (GdkInputFunction)monitor_input_callback,
			    &data);     
  attack_host(victim, -1, -1);
  gtk_main();
}

	
void monitor_list_update(data,msg)
	struct monitor_data * data;
	char * msg;
{
 char * hostname;
 int current;
 int max;
 gfloat gmax;
 gfloat gcurrent;
 GList * dlist;
 GtkObject * item;
 char * list_hostname;
 int flag = 0;
 
 
 hostname = emalloc(strlen(msg)+1);
 parse_nessusd_state(msg, hostname, &current, &max);
 dlist = GTK_LIST(data->list)->children;
 while(dlist && !flag)
 {
  item = GTK_OBJECT(dlist->data);
  list_hostname = gtk_object_get_data(item, "hostname");
  if(!list_hostname){
  	fprintf(stderr, "Error ! Null hostname in the list\n");
  	exit(1);
  	}
  if(!strcmp(list_hostname, hostname))
  {
   GtkWidget * progress_bar;
   gfloat f;
   
   progress_bar = gtk_object_get_data(item, "progress_bar");
   gmax = max;
   gcurrent = current;
   f = (gcurrent/gmax);
   if(f>1)f=1;
   if(f<0.0)f=0.0;
   gtk_progress_bar_update (GTK_PROGRESS_BAR(progress_bar), f);
   flag = 1;
  }
  dlist = dlist->next;
 }
 
 if(!flag)
 {
 /* the host was not found, we must add one... */
 monitor_add_host(data, hostname, current);
 }
 
}
 
void monitor_add_host(data,hostname,port)
	struct monitor_data * data;
	char * hostname;
	int port;
{
 GtkWidget * progress_bar;
 GtkWidget * table;
 GtkWidget * label;
 GtkWidget * item;
 GtkWidget * button;
 GtkWidget * separator;
 GList * dlist;
 gfloat f;
 
 item = gtk_list_item_new();
 dlist = NULL;
 table = gtk_table_new(2, 3, FALSE);
 gtk_table_set_col_spacings(GTK_TABLE(table), 15);
 gtk_table_set_row_spacings(GTK_TABLE(table),5);
 
 gtk_container_add(GTK_CONTAINER(item), table);
 label = gtk_label_new(hostname);
 gtk_widget_set_usize(label, 150, 15);
 gtk_table_attach_defaults(GTK_TABLE(table),label, 0,1,0,1);
 gtk_widget_show(label);
 
 progress_bar = gtk_progress_bar_new();
 gtk_table_attach_defaults(GTK_TABLE(table),progress_bar, 1,2,0,1);
 gtk_widget_show(progress_bar);
 
 button = gtk_button_new_with_label("Stop");
 gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     (GtkSignalFunc)monitor_stop_test,hostname);
 gtk_table_attach_defaults(GTK_TABLE(table),button, 2,3,0,1);
 gtk_widget_show(button);
 
 separator = gtk_hseparator_new();
 gtk_table_attach_defaults(GTK_TABLE(table), separator, 0,3,1,2);
 gtk_widget_show(separator);
 
 gtk_object_set_data(GTK_OBJECT(item), "hostname", hostname);
 gtk_object_set_data(GTK_OBJECT(item), "progress_bar", progress_bar);
 f = (gfloat)port/7001.0;
 if(f>1 || f<0.0)f=0.0;
 gtk_progress_bar_update(GTK_PROGRESS_BAR(progress_bar), f);
 gtk_widget_show(table);
 gtk_widget_show(item);
 dlist = g_list_append(dlist, item);
 gtk_list_append_items(GTK_LIST(data->list), dlist);
 }
 
void monitor_input_callback(struct monitor_data * data, int fd, int condition)
{
  int finished = 0;
  char * buf;
  char * msg;
  int type;
  buf = emalloc(4096);
  
  
  auth_gets(buf, 4095);
  buf[strlen(buf)-1]=0;
#ifdef DEBUGMORE
  fprintf(stderr, "%s\n", buf);
#endif
  
  msg = emalloc(strlen(buf)+1);
  type=parse_server_message(buf, data->hosts, msg);
  switch(type)
  {
  	case MSG_BYE : 
  		finished = 1;
  		break;
  	case MSG_STAT :
  	 	monitor_list_update(data, msg);
  		break;
  }
  
  bzero(buf, 1024);
  efree(msg);
  
  if(finished)
    {
#ifdef DEBUG
      fprintf(stderr, "Scan finished\n");
#endif
      gdk_input_remove(data->tag);
      shutdown(0,2);
      close(0);
      gtk_widget_hide(data->window);
      report_tests(data->hosts);
    }
}
 
void monitor_stop_test(GtkWidget * w, char * hostname)
{
 auth_printf("CLIENT <|> STOP_ATTACK <|> %s <|> CLIENT\n", hostname);
}
 
