/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include <config.h>
#include <nessus-devel.h>
#include <nessuslib.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <errno.h>
#include <dlfcn.h>
#include <unistd.h>
#include <dirent.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <fcntl.h>

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else   /* not HAVE_GETOPT_H */
extern int optind;
#endif /* not HAVE_GETOPT_H */   

#include "pluginload.h"
#include "preferences.h"
#include "auth.h"
#include "comm.h"
#include "attack.h"
#include "sighand.h"
#include "log.h"
#include "rules.h"
#include "threads.h"

/*
 * Global variables
 * (The less, the better)
 */

struct arglist * plugins;
struct arglist * preferences;
int serv_socket;
int serv_port;
char * config_file;

/*
 * Functions prototypes
 */
 
void main_loop();
int init_nessusd();
int init_plugins(char *);
int init_network(int);


void main_loop()
{
  while(1)
    {
      int soc;
      unsigned int lg_address = sizeof(struct sockaddr_in);
      struct sockaddr_in address;
      
      soc = accept(serv_socket, &address, &lg_address);
      if(soc == -1){
    	if(errno == EINTR)continue;
  	else {
	  perror("accept ");
	  exit(2);
	}
      }
      log_write("connection from %s\n", (char *)inet_ntoa(address.sin_addr));
      if(fork()==0)
	{
	  comm_init(soc);
	  if(auth_check_user((char *)inet_ntoa(address.sin_addr)))
	    {
	    auth_printf("Bad login attempt !\n"); 
	    log_write("bad login attempt from %s\n", 
	    	(char *)inet_ntoa(address.sin_addr));
	    }
	  else {
	    comm_send_pluginlist();
	    attack_host(comm_wait_order());
	    comm_terminate();
	  }
	  close(serv_socket);
	  shutdown(soc, 2);
	  close(soc);
	  exit(0);
	}
    }
}


/*
 * Initialization of the network : 
 * we setup the socket that will listen for
 * incoming connections on port <port>
 */ 
int init_network(port)
     int port;
{
  struct sockaddr_in address;
  
  if((serv_socket = socket(AF_INET, SOCK_STREAM, 0))==-1)
    {
      perror("socket ");
      exit(1);
    }
  address.sin_family = AF_INET;
  address.sin_addr.s_addr = htonl(INADDR_ANY);
  address.sin_port = htons(port);
  if(bind(serv_socket, &address, sizeof(struct sockaddr_in))==-1)
    {
      perror("bind ");
      exit(1);
    }
  if(listen(serv_socket, 10)==-1)
    {
      perror("listen ");
      exit(1);
    }
  return(0);
}

/*
 * Initialize everything
 */
int init_nessusd()
{
  preferences_init(&preferences);
  rules_init();
  nessusd_threads_init();
  log_init(arg_get_value(preferences, "logfile"));
  plugins_init(arg_get_value(preferences, "plugins_folder"));
  init_network(serv_port);
  signal(SIGCHLD, sighand_chld);
  signal(SIGTERM, sighand_term);
  signal(SIGSEGV, sighand_segv);
  return(0);
}


void display_help()
{
 	printf("nessusd, version %s\n", NESSUS_VERSION);
    	printf("\nusage : nessusd [-vcph]\n\n");
    	printf(" v : show version number\n h : show this help\n");
    	printf(" p < number > : use port number <number>\n");
    	printf(" c <filename> : alternate configuration file to use (default : %s)\n",
    		NESSUSD_CONF);
}


void main(int argc, char * argv[])
{
  int i;
  
  serv_port = -1;
  config_file = NULL;
  while((i=getopt(argc, argv, "phvc"))!=EOF)
  {
   switch(i)
   {
    case 'p' :
    	if(!argv[optind]){display_help();exit(0);}
    	serv_port = atoi(argv[optind]);
    	if((serv_port<=0)||(serv_port>=65536))
    		{
    		printf("Invalid port specification\n");
    		display_help();
    		exit(1);
    		}
    	break;
    	
    case 'v' :
    	printf("nessusd (%s) %s -- (c) 1998 Renaud Deraison\n", 
    			PROGNAME,NESSUS_VERSION);
    	exit(0);
    	break;
    case 'h' :
    	display_help();
    	exit(0);
    	break;
    case 'c' : 
    	if(!argv[optind]){display_help();exit(1);}
    	config_file = emalloc(strlen(argv[optind])+1);
    	strncpy(config_file, argv[optind], strlen(argv[optind]));
    	break;
    }
  } 
  if(serv_port == -1)serv_port = DEFAULT_PORT;
  if(!config_file)
  {
  	config_file = emalloc(strlen(NESSUSD_CONF)+1);
  	strncpy(config_file, NESSUSD_CONF, strlen(NESSUSD_CONF));
  }
  init_nessusd();
  main_loop();
  exit(0);
}
 
 
