/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Plugins Loader
 *
 */
 
#include <config.h>
#include <nessus-devel.h>
#include <nessuslib.h>
#include <stdio.h>
#include <unistd.h>
#include <dirent.h>
#include <dlfcn.h>
#include "pluginload.h"
#include "log.h"

extern struct arglist * plugins;
extern struct arligst * preferences;


/*
 * main function for loading all the
 * plugins that are in folder <folder>
 */
int plugins_init(folder)
	char * folder;
{
  DIR * dir;
  struct dirent * dp;
  
  if(!folder)
  {
#ifdef DEBUG
   log_write("%s:%d : folder == NULL\n", __FILE__, __LINE__);
#endif
   printf("could not determine the value of <plugins_folder>. Check "NESSUSD_CONF"\n");
   exit(0);
  }
  dir = opendir(folder);
  if(dir == NULL)
    {
      log_write("Couldn't open the repertory called \"%s\"\n", folder);
      exit(1);
    }
  
  if(!plugins)
  {
   plugins = emalloc(sizeof(struct arglist));
  }
  while((dp = readdir(dir)))
    if (!strcmp(".nes", (dp->d_name+(strlen(dp->d_name)-4))))
      plugin_add(folder, dp->d_name);
#ifdef DEBUG
  printf("Plugins initialized\n");
#endif  
  return(0);
}

/*
 * add *one* plugin to the server list
 */
void plugin_add(folder, name)
	char * folder;
	char * name;
{
  void * ptr = NULL;
  char * fullname = emalloc(strlen(folder)+strlen(name)+2);
  int mode;
  
  sprintf(fullname, "%s/%s", folder, name);
#ifdef DEBUGMORE
  printf("Attempting to load %s (%s)\n", name, fullname);
#endif

#ifdef RTLD_NOW
  mode = RTLD_NOW;
#else
  mode = 1;	/* RTLD_LAZY... */
#endif
  if((ptr = dlopen(fullname, mode))==NULL){
    log_write("Couldn't load %s\n", name);
    log_write("Error : %s\n", dlerror());
  }
  else {
    struct arglist * plugin;
    plugin_init_t  f_init;
    plugin_run_t  f_run;
    struct arglist * p_args;
    plugin = emalloc(sizeof(struct arglist));
    
    if((f_init = (plugin_init_t)dlsym(ptr, "plugin_init")) &&
    	(f_run = (plugin_run_t)dlsym(ptr, "plugin_run")))
      {
      	p_args = emalloc(sizeof(struct arglist));
      	arg_add_value(p_args, "preferences", ARG_ARGLIST, -1, (void*)preferences);
#ifdef DEBUGMORE
	printf("Calling plugin_init() (%d) \n", (int)f_init);
#endif
      	(*f_init)(p_args);
#ifdef DEBUG 
	printf("Adding \"%s\" in the plugins list\n", name);
#endif
 	arg_add_value(plugin, "plugin_init", ARG_PTR, -1, (void *)f_init);
 	arg_add_value(plugin, "plugin_run", ARG_PTR, -1, (void *)f_run);
 	arg_add_value(plugin, "plugin_args", ARG_ARGLIST, -1, (void *)p_args);
 	arg_add_value(plugins, name, ARG_ARGLIST, -1, (void *)plugin);
      }
    else log_write("Couldn't find the entry point in %s [%s]\n", name, dlerror());
  }
}

