/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */           
 
#include <config.h>
#include <nessus-devel.h>
#include <nessuslib.h>
#include <stdlib.h>
#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <unistd.h>
#include <pwd.h>
#include <fcntl.h>
#include <sys/mman.h>
#ifdef HAVE_FNMATCH
#include <fnmatch.h>
#endif

#include "rules.h"

extern struct arglist * preferences;
struct nessus_rules * rules;


char * rules_get_fname()
{
  char * t;
  if((t=arg_get_value(preferences, "rules")))return(t);
  else return(NESSUSD_RULES);
}


void rules_init()
{
#ifdef HAVE_FNMATCH
  FILE * fd;
  struct nessus_rules * p ;
  char * fn = rules_get_fname();
  
  p = rules = emalloc(sizeof(struct nessus_rules));
  fd = fopen(fn, "r"); 
  if(!fd)
    {
      char * buf = emalloc(200);
      int f = open(fn, O_CREAT | O_RDWR,  0660);
      
      chown(fn, getuid(), getgid());
      close(f);
      f = open(fn, O_RDWR);
      sprintf(buf, 
     "#Nessusd rules\n#cd doc/sample.rules for details\nn:localhost.*\n");
      write(f, buf, strlen(buf));
      efree(buf);
      close(f);
      p->name = emalloc(12);
      sprintf(p->name, "localhost.*");
      p->category = p_DONT_TEST;
      p->next = NULL;
    }
  else
    {
      char * buf = emalloc(255);
      
      bzero(buf, 255);
      while(fgets(buf, 255, fd) && !feof(fd))
	{
	  if(buf[0]!='#' && strlen(buf)>2)
	    {
	      switch(buf[0])
		{
		case 'n':
		  p->category = p_DONT_TEST;
		  break;
		case 'd':
		  p->category = p_DONT_DNS;
		  break;
		case 's':
		  p->category = p_DONT_SM;
		  break;
		case 'y':
		  p->category = p_DO_TEST;
		  break;
		default : 
		  p->category = -1;
		  fprintf(stderr, "Parse error in %s : \"%s\"\n",fn,  buf);
		}
	      if(p->category != -1)
		{
		  buf+=2;
		  if(buf[strlen(buf)-1]=='\n')buf[strlen(buf)-1]=0;
		  p->name = emalloc(strlen(buf) + 1);
		  sprintf(p->name, "%s", buf);
		  p->next = emalloc(sizeof(struct nessus_rules));
		  bzero(p->next, sizeof(struct nessus_rules));
		  p = p->next;
		  buf-=2;
		}
	    }
	   bzero(buf, 255);
	}
      efree(buf);
      fclose(fd);
    }
#endif
}



struct nessus_rules * rules_parse(name, r)
   char * name;
   struct nessus_rules * r;
{
#ifdef HAVE_FNMATCH
  char good = 1;
  
  while(r && r->name && good)
    {
      if(!fnmatch(r->name, name, 0))good = 0;
      else r = r->next;
    }
  return(good ? NULL:r);
#else
  return(p_DO_TEST);
#endif
}


short get_hostname_attributes(name)
  char * name;
{
#ifdef HAVE_FNMATCH
  struct nessus_rules * p = rules;
  short a=0;
  
  while(p && p->name)
   {
    p = rules_parse(name, p);
    if(p)
    {
    a += (a & p->category)?0:p->category;
    if(p)p = p->next;
    }
   }
  return(a);
#else
  return(p_DO_TEST);
#endif
} 

	
		
	
	  
